--=======================================================================================================================
print("Billah Decisions: loaded")
--=======================================================================================================================
-- UTILITIES
--=======================================================================================================================
-- MOD CHECKS
-------------------------------------------------------------------------------------------------------------------------
-- JFD_IsUsingCPDLL
function JFD_IsUsingCPDLL()
	local cPDLLID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if (mod.ID == cPDLLID) then
			return true
		end
	end
	return false
end
local isUsingCPDLL = JFD_IsUsingCPDLL()
-------------------------------------------------------------------------------------------------------------------------
-- UTILITIES
-------------------------------------------------------------------------------------------------------------------------
-- JFD_SendWorldEvent
function JFD_SendWorldEvent(playerID, description)
	local player = Players[playerID]
	local playerTeam = Teams[player:GetTeam()]
	local activePlayer = Players[Game.GetActivePlayer()]
	if (not player:IsHuman()) and playerTeam:IsHasMet(activePlayer:GetTeam()) then
		Players[Game.GetActivePlayer()]:AddNotification(NotificationTypes["NOTIFICATION_DIPLOMACY_DECLARATION"], description, "[COLOR_POSITIVE_TEXT]World Events[ENDCOLOR]", -1, -1)
	end
end 
--=======================================================================================================================
-- DECISIONS
--=======================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local civilizationID  = GameInfoTypes["CIVILIZATION_THP_FATIMID_BILLAH"]
local mathCeil = math.ceil

-------------------------------------------------------------------------------------------------------------------------
-- Billah: Preach in Enemy Lands
-------------------------------------------------------------------------------------------------------------------------
local epicID = GameInfoTypes.BUILDING_NATIONAL_EPIC
local holySite = GameInfoTypes.IMPROVEMENT_HOLY_SITE
local policyHolySite = GameInfoTypes.POLICY_DECISIONS_THP_BILLAH_PREACHING

function IsEnemyUnhappy(player)
    local pTeam = Teams[player:GetTeam()]
    for otherTeamID, otherTeam in pairs(Teams) do
        if pTeam:IsAtWar(otherTeamID) and (not otherTeam:IsBarbarian()) then
            local pOther = Players[otherTeam:GetLeaderID()]
            if pOther:GetHappiness() < 0 then
                return pOther
            end
        end
    end
    return nil
end

function LatestInfantry(player)
    local latestUnit = nil
    local team = Teams[player:GetTeam()]
    if team:IsHasTech(GameInfoTypes.TECH_MOBILE_TACTICS) then
        latestUnit = GameInfoTypes.UNIT_MECHANIZED_INFANTRY
    elseif team:IsHasTech(GameInfoTypes.TECH_PLASTIC) then
        latestUnit = GameInfoTypes.UNIT_INFANTRY
    elseif team:IsHasTech(GameInfoTypes.TECH_REPLACEABLE_PARTS) then
        latestUnit = GameInfoTypes.UNIT_GREAT_WAR_INFANTRY
    elseif team:IsHasTech(GameInfoTypes.TECH_RIFLING) then
        latestUnit = GameInfoTypes.UNIT_RIFLEMAN
    elseif team:IsHasTech(GameInfoTypes.TECH_GUNPOWDER) then
        latestUnit = GameInfoTypes.UNIT_MUSKETMAN
    elseif team:IsHasTech(GameInfoTypes.TECH_CIVIL_SERVICE) then
        latestUnit = GameInfoTypes.UNIT_PIKEMAN
    elseif team:IsHasTech(GameInfoTypes.TECH_BRONZE_WORKING) then
        latestUnit = GameInfoTypes.UNIT_SPEARMAN
    else
        latestUnit = GameInfoTypes.UNIT_WARRIOR
    end
end

local Decisions_THP_Billah_Preaching = {}
	Decisions_THP_Billah_Preaching.Name = "TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_PREACHING"
	Decisions_THP_Billah_Preaching.Desc = "TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_PREACHING_DESC"
	HookDecisionCivilizationIcon(Decisions_THP_Billah_Preaching, "CIVILIZATION_THP_FATIMID_BILLAH")
	Decisions_THP_Billah_Preaching.CanFunc = (
	function(player)
		if player:GetCivilizationType() ~= civilizationID then return false, false end
        local iEra = load(player, "Decisions_THP_Billah_Preaching")
        local iCurrentEra = player:GetCurrentEra()
        if iEra ~= nil then
            if iEra < iCurrentEra then
                save(player, "Decisions_THP_Billah_Preaching", nil)
            else
                Decisions_THP_Billah_Preaching.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_PREACHING_ENACTED_DESC")
                return false, false, true
            end
        end
        local cultureCost = mathCeil(200*iMod)
		Decisions_THP_Billah_Preaching.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_PREACHING_DESC", cultureCost)
		if player:GetJONSCulture() < cultureCost then return true, false end
		if IsEnemyUnhappy(player) == nil then return true, false end
		if player:GetNumResourceAvailable(iMagistrate, false) < 1 then return true, false end
		return true, true
	end
	)

	Decisions_THP_Billah_Preaching.DoFunc = (
	function(player)
		local playerID = player:GetID()
		local cultureCost = mathCeil(200*iMod)
		local capital = player:GetCapitalCity()
		player:ChangeNumResourceTotal(iMagistrate, -1)
		player:ChangeJONSCulture(-cultureCost)
        save(player, "Decisions_THP_Billah_Preaching", true)
        local pEnemy = IsEnemyUnhappy(player)
        local pEnemyCap = pEnemy:GetCapitalCity()
        local barbPlayer = nil
        for nextPlayerID, nextPlayer in pairs(Players) do
            if nextPlayer:IsBarbarian() then
                barbPlayer = nextPlayer
            end
        end
        barbPlayer:InitUnit(LatestInfantry(pEnemy), (pEnemyCap:GetX() - 1), (pEnemyCap:GetY() - 1))
        if pEnemy:GetHappiness() < -2 then
            barbPlayer:InitUnit(LatestInfantry(pEnemy), (pEnemyCap:GetX() + 1), (pEnemyCap:GetY() + 1))
        end
        if pEnemy:GetHappiness() < -4 then
            barbPlayer:InitUnit(LatestInfantry(pEnemy), (pEnemyCap:GetX() + 1), (pEnemyCap:GetY() - 1))
        end
		JFD_SendWorldEvent(playerID, Locale.ConvertTextKey("TXT_KEY_WORLD_EVENT_THP_FATIMID_BILLAH_PREACHING", player:GetName(), player:GetCivilizationShortDescription(), player:GetCivilizationAdjective()))
	end
	)
	
Decisions_AddCivilisationSpecific(civilizationID, "Decisions_THP_Billah_Preaching", Decisions_THP_Billah_Preaching)

-------------------------------------------------------------------------------------------------------------------------
-- Billah: Build a Raiding Fleet
-------------------------------------------------------------------------------------------------------------------------
local eraMedievalID = GameInfoTypes.ERA_MEDIEVAL
local uShini = GameInfoTypes.UNIT_THP_SHINI
local pVolley = GameInfoTypes.PROMOTION_VOLLEY
local pBomb = GameInfoTypes.PROMOTION_BOMBARDMENT_1

local Decisions_THP_Billah_Raiding = {}
	Decisions_THP_Billah_Raiding.Name = "TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_RAIDING"
	Decisions_THP_Billah_Raiding.Desc = "TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_RAIDING_DESC"
	HookDecisionCivilizationIcon(Decisions_THP_Billah_Raiding, "CIVILIZATION_THP_FATIMID_BILLAH")
	Decisions_THP_Billah_Raiding.CanFunc = (
	function(player)
		if player:GetCivilizationType() ~= civilizationID then return false, false end
		if load(player, "Decisions_THP_Billah_Raiding") == true then
			Decisions_THP_Billah_Raiding.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_RAIDING_ENACTED_DESC")
			return false, false, true
		end
		local goldCost = mathCeil(200*iMod)
		Decisions_THP_Billah_Raiding.Desc = Locale.ConvertTextKey("TXT_KEY_DECISIONS_THP_FATIMID_BILLAH_RAIDING_DESC", goldCost)
		if player:GetGold() < goldCost then return true, false end
		if player:GetCurrentEra() < eraMedievalID then return true, false end
		if player:GetNumResourceAvailable(iMagistrate, false) < 1 then return true, false end
		return true, true
	end
	)

	Decisions_THP_Billah_Raiding.DoFunc = (
	function(player)
		local playerID = player:GetID()
		local goldCost = mathCeil(200*iMod)
		player:ChangeNumResourceTotal(iMagistrate, -1)
		player:ChangeGold(-goldCost)
		save(player, "Decisions_THP_Billah_Raiding", true)
		local cFirstCoast = nil
		for pCity in player:Cities() do
			if pCity:IsCoastal() then
				cFirstCoast = pCity
				break
			end
		end
		if not cFirstCoast then
			cFirstCoast = player:GetCapitalCity()
		end
		player:InitUnit(uShini, cFirstCoast:GetX(), cFirstCoast:GetY())
		local plot = cFirstCoast:Plot()
		local iNumUnits = plot:GetNumUnits()
		for iVal = 0,(iNumUnits - 1) do
			local pHereUnit = plot:GetUnit(iVal)
			if pHereUnit:GetUnitType() == uShini then
				pHereUnit:SetHasPromotion(pVolley, true)
				pHereUnit:SetHasPromotion(pBomb, true)
			end
		end
		JFD_SendWorldEvent(playerID, Locale.ConvertTextKey("TXT_KEY_WORLD_EVENT_THP_FATIMID_BILLAH_RAIDING", player:GetName(), player:GetCivilizationShortDescription(), player:GetCivilizationAdjective()))
	end
	)
	
Decisions_AddCivilisationSpecific(civilizationID, "Decisions_THP_Billah_Raiding", Decisions_THP_Billah_Raiding)
