-- ========= --
-- UTILITIES --
-- ========= --

function IsCivilizationActive(civilizationID)
	for iSlot = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotStatus = PreGame.GetSlotStatus(iSlot)
		if (slotStatus == SlotStatus["SS_TAKEN"] or slotStatus == SlotStatus["SS_COMPUTER"]) then
			if PreGame.GetCivilization(iSlot) == civilizationID then
				return true
			end
		end
	end
	return false
end

function JFDGame_IsAAActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "432bc547-eb05-4189-9e46-232dbde8f09f" then
			return true
		end
	end
	return false
end
local isAAActive = JFDGame_IsAAActive()

-- ======= --
-- DEFINES --
-- ======= --

include("AdditionalAchievementsUtility.lua");
include("PlotIterators.lua");

local pCiv = GameInfoTypes.CIVILIZATION_THP_FATIMID_BILLAH
local bIsActive = local bIsActive = IsCivilizationActive(pCiv)

-- ====================================== --
-- UA: MAY CONVERT ENEMIES KILLED NEAR GP --
-- ====================================== --

local pCapturePromo = GameInfoTypes.PROMOTION_THP_DAI_CONVERT

function UpdateCaptureStatus(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:GetCivilizationType() ~= pCiv then return end
	for uOrigUnit in pPlayer:Units() do
		local bIsItStacked = false
		if uOrigUnit:IsCombatUnit() then
			local pPlot = uOrigUnit:GetPlot()
			local iNumUnits = pPlot:GetNumUnits()
			if iNumUnits > 1 then
				for iVal = 0,(iNumUnits - 1) do
					local uCheckedUnit = pPlot:GetUnit(iVal)
					if uCheckedUnit:IsGreatPerson() then
						bIsItStacked = true
					end
				end
			end
		end
		uOrigUnit:SetHasPromotion(pCapturePromo, bIsItStacked)
	end
end

if bIsActive then
	GameEvents.PlayerDoTurn.Add(UpdateCaptureStatus)
end

-- ================== --
-- UA: GP MOVE FASTER --
-- ================== --

local pFastGP = GameInfoTypes.PROMOTION_THP_FAST_GP

-- This function was originally written by ViceVirtuoso for CL's Boers
function PromoteGP(iPlayer, iUnitID)
	if iPlayer < GameDefines.MAX_MAJOR_CIVS then
		local pPlayer = Players[iPlayer]
		if pPlayer:GetCivilizationType() == pCiv then
			local pUnit = pPlayer:GetUnitByID(iUnitID)
			if pUnit:IsGreatPerson() and not pUnit:IsHasPromotion(pFastGP) then
				pUnit:SetHasPromotion(pFastGP, true)
				local iNumMoves = (pUnit:GetMoves() + 2) * 60
				pUnit:SetMoves(iNumMoves)
			end
		end
	end
end

if bIsActive then
	GameEvents.UnitSetXY.Add(PromoteGP)
end

-- ==================================== --
-- UU1: KUTAMA STRONGER STACKED WITH GP --
-- ==================================== --

local uKutama = GameInfoTypes.UNIT_THP_KUTAMA
local uKutamaClass = GameInfoTypes.UNITCLASS_KNIGHT
local pRegularPromo = GameInfoTypes.PROMOTION_THP_KUTAMA
local pDummyPromo = GameInfoTypes.PROMOTION_THP_KUTAMA_DUMMY

local tGreatPeople = {}
local iTableNum = 1
for row in DB.Query("SELECT Class FROM Units WHERE Special = 'SPECIALUNIT_PEOPLE'") do
	tGreatPeople[iTableNum] = row.Class
	iTableNum = iTableNum + 1
end

function HasGreatPerson(pPlayer)
	for k, vClass in pairs(tGreatPeople) do
		if pPlayer:HasUnitOfClassType(vClass) then
			return true
		end
	end
	return false
end

function CheckKutama(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:HasUnitOfClassType(uKutamaClass) and HasGreatPerson(pPlayer) then
		for uUnit in pPlayer:Units() do
			if uUnit:GetUnitType() == uKutama then
				local isStackedWithGP = false
				local pUnitPlot = uUnit:GetPlot()
				local iNumUnits = pUnitPlot:GetNumUnits()
				for iVal = 0, (iNumUnits - 1), 1 do
					local pStackedUnit = pUnitPlot:GetUnit(iVal)
					if pStackedUnit:IsGreatPerson() then
						isStackedWithGP = true
					end
				end
				if isStackedWithGP then
					uUnit:SetHasPromotion(pDummyPromo, true)
					uUnit:SetHasPromotion(pRegularPromo, false)
				else
					uUnit:SetHasPromotion(pDummyPromo, false)
					uUnit:SetHasPromotion(pRegularPromo, true)
				end
			end
		end
	end
end

if bIsActive then
	GameEvents.PlayerDoTurn.Add(CheckKutama)
end

-- ========== --
-- AA SUPPORT --
-- ========== --

function BillahAACheck(ownerID, religionID, iX, iY)
	if IsAAUnlocked('AA_THP_BILLAH_SPECIAL') then return end
	local pCityOwner = Players[ownerID]
	local tOwningTeam = Teams[pCityOwner:GetTeam()]
	local cConvertedCity = Map.GetPlot(iX, iY):GetPlotCity()
	if cConvertedCity:IsHolyCityAnyReligion() then
		if tOwningTeam:GetAtWarCount(false) > 0 then
			for otherTeamID, otherTeam in pairs(Teams) do
				if tOwningTeam:IsAtWar(otherTeamID) then
					local otherPlayer = Players[otherTeam:GetLeaderID()]
					if (otherPlayer:GetCivilizationType() == pCiv) and (otherPlayer:IsHuman()) then
						UnlockAA('AA_THP_BILLAH_SPECIAL');
					end
				end
			end
		end
	end
end

if isAAActive then
	GameEvents.CityConvertsReligion.Add(BillahAACheck)
end
