-- ========= --
-- UTILITIES --
-- ========= --

function IsCivilizationActive(civilizationID)
	for iSlot = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotStatus = PreGame.GetSlotStatus(iSlot)
		if (slotStatus == SlotStatus["SS_TAKEN"] or slotStatus == SlotStatus["SS_COMPUTER"]) then
			if PreGame.GetCivilization(iSlot) == civilizationID then
				return true
			end
		end
	end
	return false
end

function JFDGame_IsAAActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "432bc547-eb05-4189-9e46-232dbde8f09f" then
			return true
		end
	end
	return false
end
local isAAActive = JFDGame_IsAAActive()

-- ======= --
-- DEFINES --
-- ======= --

include("AdditionalAchievementsUtility.lua");
include("FLuaVector.lua");

local pCiv = GameInfoTypes.CIVILIZATION_THP_FATIMID_MUIZZ
local bIsActive = IsCivilizationActive(pCiv)

-- ================================== --
-- UA: GS POINTS FROM FAITH PURCHASES --
-- ================================== --

local sScientist = GameInfo.Specialists.SPECIALIST_SCIENTIST.ID

function PointsFromUnits(playerID, cityID, unitID, bGold, bFaith)
	local pPlayer = Players[playerID]
	if pPlayer:GetCivilizationType() == pCiv then
		if bFaith then
			local pCity = pPlayer:GetCityByID(cityID)
			local pUnit = pPlayer:GetUnitByID(unitID)
			if pUnit:IsGreatPerson() then
				-- to buy GP w/ Faith, you need to already be in the Industrial Era...
				-- ...so with any luck this will be balanced
				pCity:ChangeSpecialistGreatPersonProgressTimes100(sScientist, 5000)
			else
				pCity:ChangeSpecialistGreatPersonProgressTimes100(sScientist, 1500)
			end
		end
	end
end

function PointsFromBuildings(playerID, cityID, buildingType, bGold, bFaith)
	local pPlayer = Players[playerID]
	if pPlayer:GetCivilizationType() == pCiv then
		if bFaith then
			local pCity = pPlayer:GetCityByID(cityID)
			pCity:ChangeSpecialistGreatPersonProgressTimes100(sScientist, 2500)
		end
	end
end

if bIsActive then
	GameEvents.CityTrained.Add(PointsFromUnits)
	GameEvents.CityConstructed.Add(PointsFromBuildings)
end

-- ======================================= --
-- UA: SCIENCE FROM EXPENDING RELIG. UNITS --
-- ======================================= --

local uMissionary = GameInfoTypes.UNIT_MISSIONARY
local uInquisitor = GameInfoTypes.UNIT_INQUISITOR

local gsQuick = GameInfoTypes.GAMESPEED_QUICK
local gsEpic = GameInfoTypes.GAMESPEED_EPIC
local gsMarathon = GameInfoTypes.GAMESPEED_MARATHON

-- This function is heavily derived from the Seminary Lua for JFD's Spain

function ScienceBurstQuantity(player)
    local scienceBurst = (player:GetScience())
    if Game.GetGameSpeedType() == gsQuick then
        scienceBurst = (2 * scienceBurst) / 3
    elseif Game.GetGameSpeedType() == gsEpic then
        scienceBurst = 1.5 * scienceBurst
    elseif Game.GetGameSpeedType() == gsMarathon then
        scienceBurst = 3 * scienceBurst
    end
    return scienceBurst
end

function MissionScience(playerID, unitID, unitType, plotX, plotY, isDelay, otherPlayerID)
	local player = Players[playerID]
	if not isDelay then return end
	if not player:IsAlive() then return end
	if otherPlayerID ~= playerID then return end
	if player:GetCivilizationType() == pCiv then
		local scienceBurst = 0
		if (unitType == uMissionary) or (unitType == uInquisitor) then
        		scienceBurst = ScienceBurstQuantity(player)
			if player:IsHuman() then
				Events.AddPopupTextEvent(HexToWorld(ToHexFromGrid(Vector2(plotX, plotY))), "[COLOR_RESEARCH_STORED]+" .. scienceBurst .. " [ICON_RESEARCH] Science[ENDCOLOR]", 0)
			end
			local pTeam = Teams[player:GetTeam()]
			local tCurrentTech = player:GetCurrentResearch()
			pTeam:GetTeamTechs():ChangeResearchProgress(tCurrentTech, scienceBurst, player)
		end
	end
end

if bIsActive then
	GameEvents.UnitPrekill.Add(MissionScience)
end

-- ============================================== --
-- UU: CALCULATE AT-WAR STATUS WITH RELIG. RIVALS --
-- ============================================== --

function DoYouShareReligions(pFatimidPlayer, pOtherPlayer)
	local pYourReligion = nil
	local pOtherReligion = nil
	if pFatimidPlayer:HasCreatedReligion() then
		pYourReligion = pFatimidPlayer:GetReligionCreatedByPlayer()
	else
		if not pFatimidPlayer:GetCapitalCity() then
			pYourReligion = -1
        	else
			pYourReligion = pFatimidPlayer:GetCapitalCity():GetReligiousMajority()
        	end
	end
	if pOtherPlayer:HasCreatedReligion() then
		pOtherReligion = pOtherPlayer:GetReligionCreatedByPlayer()
	else
		if not pOtherPlayer:GetCapitalCity() then
			pOtherReligion = -1
		else
			pOtherReligion = pOtherPlayer:GetCapitalCity():GetReligiousMajority()
		end
	end
	if pYourReligion == pOtherReligion then
		return true
	else
		return false
	end
end

function IsAtWarWithReligiousRival(player)
	local tYourTeam = Teams[player:GetTeam()]
	local isWar = false
	for otherTeamID, otherTeam in pairs(Teams) do
		if tYourTeam:IsAtWar(otherTeamID) then
			if not otherTeam:IsBarbarian() then
				local pOther = Players[otherTeam:GetLeaderID()]
				if not DoYouShareReligions(player, pOther) then
					isWar = true
					return isWar
				end
			end
		end
	end
	return isWar
end

-- ==================================================== --
-- UU: UPDATE AHDATH COMBAT STRENGTH BASED ON WAR/PEACE --
-- ==================================================== --

local cClass = GameInfoTypes.UNITCLASS_CROSSBOWMAN
local uAhdath = GameInfoTypes.UNIT_THP_AHDATH
local pIsAtWar = GameInfoTypes.POLICY_THP_AHDATH_AT_WAR

function AhdathPromote(declarerID, declaredUponID)
	local tDeclarer = Teams[declarerID]
	local pDeclarer = Players[tDeclarer:GetLeaderID()]
	local tDeclaredUpon = Teams[declaredUponID]
	local pDeclaredUpon = Players[tDeclaredUpon:GetLeaderID()]
	if pDeclarer:HasUnitOfClassType(cClass) then
		if not pDeclarer:HasPolicy(pIsAtWar) then
			if not DoYouShareReligions(pDeclarer, pDeclaredUpon) then
				pDeclarer:SetNumFreePolicies(1)
				pDeclarer:SetNumFreePolicies(0)
				pDeclarer:SetHasPolicy(pIsAtWar, true)
				for pUnit in pDeclarer:Units() do
					if pUnit:GetUnitType() == uAhdath then
						local iStrength = pUnit:GetBaseCombatStrength()
						pUnit:SetBaseCombatStrength(iStrength + 3)
					end
				end
			end
		end
	end
	-- this isn't an elseif, in case there are two Fatimids in the game and they go to war with each other
	if pDeclaredUpon:HasUnitOfClassType(cClass) then
		if not pDeclaredUpon:HasPolicy(pIsAtWar) then
			if not DoYouShareReligions(pDeclaredUpon, pDeclarer) then
				pDeclaredUpon:SetNumFreePolicies(1)
				pDeclaredUpon:SetNumFreePolicies(0)
				pDeclaredUpon:SetHasPolicy(pIsAtWar, true)
				for pUnit in pDeclaredUpon:Units() do
					if pUnit:GetUnitType() == uAhdath then
						local iStrength = pUnit:GetBaseCombatStrength()
						pUnit:SetBaseCombatStrength(iStrength + 3)
					end
				end
			end
		end
	end
end

function AhdathDemote(peacerID, peacedUponID)
	local tPeacer = Teams[peacerID]
	local pPeacer = Players[tPeacer:GetLeaderID()]
	local tPeacedUpon = Teams[peacedUponID]
	local pPeacedUpon = Players[tPeacedUpon:GetLeaderID()]
	if pPeacer:HasPolicy(pIsAtWar) then
		local endOfReligiousWar = true
		if IsAtWarWithReligiousRival(pPeacer) then
			endOfReligiousWar = false
		end
		if endOfReligiousWar then
			pPeacer:SetHasPolicy(pIsAtWar, false)
			if pPeacer:HasUnitOfClassType(cClass) then
				for pUnit in pPeacer:Units() do
					if pUnit:GetUnitType() == uAhdath then
						local iStrength = pUnit:GetBaseCombatStrength()
						pUnit:SetBaseCombatStrength(iStrength - 3)
					end
				end	
			end
		end
	end
	if pPeacedUpon:HasPolicy(pIsAtWar) then
		local endOfReligiousWar = true
		if IsAtWarWithReligiousRival(pPeacedUpon) then
			endOfReligiousWar = false
		end
		if endOfReligiousWar then
			pPeacedUpon:SetHasPolicy(pIsAtWar, false)
			if pPeacedUpon:HasUnitOfClassType(cClass) then
				for pUnit in pPeacedUpon:Units() do
					if pUnit:GetUnitType() == uAhdath then
						local iStrength = pUnit:GetBaseCombatStrength()
						pUnit:SetBaseCombatStrength(iStrength - 3)
					end
				end	
			end
		end
	end
end

-- updates the strength of an ahdath trained *during* an ongoing religious war
function PrepNewAhdath(playerID, cityID, unitID)
	local pPlayer = Players[playerID]
	local uNewUnit = pPlayer:GetUnitByID(unitID)
	if uNewUnit:GetUnitType() == uAhdath then
		if IsAtWarWithReligiousRival(pPlayer) then
			local iStrength = uNewUnit:GetBaseCombatStrength()
			uNewUnit:SetBaseCombatStrength(iStrength + 3)
		end
	end
end

if bIsActive then
	GameEvents.DeclareWar.Add(AhdathPromote)
	GameEvents.MakePeace.Add(AhdathDemote)
	GameEvents.CityTrained.Add(PrepNewAhdath)
end

-- ============================ --
-- UB: FAITH PER WORKED ACADEMY --
-- ============================ --

local bMazar = GameInfoTypes.BUILDING_THP_MAZAR
local pAcademy = GameInfoTypes.IMPROVEMENT_ACADEMY

function MazarCheck(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:CountNumBuildings(bMazar) > 0 then
		local faithUpdate = 0
		for pCity in pPlayer:Cities() do
			if pCity:IsHasBuilding(bMazar) then
				for i = 0, pCity:GetNumCityPlots(), 1 do
					local pCheckedPlot = pCity:GetCityIndexPlot(i)
					if pCheckedPlot:GetImprovementType() == pAcademy then
						faithUpdate = faithUpdate + 2
					end
				end
			end
		end
		pPlayer:ChangeFaith(faithUpdate)
	end
end

if bIsActive then
	GameEvents.PlayerDoTurn.Add(MazarCheck)
end

-- ========== --
-- AA SUPPORT --
-- ========== --

local pSciPolicy1 = GameInfoTypes.POLICY_AA_THP_MUIZZ_SCI_DUMMY_1
local pSciPolicy2 = GameInfoTypes.POLICY_AA_THP_MUIZZ_SCI_DUMMY_2
local uScientist = GameInfoTypes.UNIT_SCIENTIST

function MuizzAACheck(playerID, cityID, uUnitType, bGold, bFaith)
	if IsAAUnlocked('AA_THP_MUIZZ_SPECIAL') then return end
	local pPlayer = Players[playerID]
	if not pPlayer:IsHuman() then return end
	if pPlayer:GetCivilizationType() == pCiv then
		if bFaith and (uUnitType == uScientist) then
			if pPlayer:HasPolicy(pSciPolicy1) then
				if pPlayer:HasPolicy(pSciPolicy2) then
					UnlockAA('AA_THP_MUIZZ_SPECIAL');
				else
					pPlayer:SetNumFreePolicies(1)
					pPlayer:SetNumFreePolicies(0)
					pPlayer:SetHasPolicy(pSciPolicy2, true)
				end
			else
				pPlayer:SetNumFreePolicies(1)
				pPlayer:SetNumFreePolicies(0)
				pPlayer:SetHasPolicy(pSciPolicy1, true)
			end
		end
	end
end

if isAAActive then
	GameEvents.CityTrained.Add(MuizzAACheck)
end

